<?php

namespace UBRIR;

/**
 * Магические обертки вокруг GetOrders()
 * @method ShowOrders($pageNum, $perPage) Вывод списка заказов в html виде
 * @method ReturnHTMLOrders($pageNum, $perPage) Возвращение html шаблона списка заказов
 * @method JsonOrders($pageNum, $perPage) Списка заказов в json формате
 * @method HtmlInJsonOrders($pageNum, $perPage) Списков заказов в json в том числе шаблон внутри JSON
 *
 * Магические обертки вокруг GetOrderStatus()
 * @method ShowOrderStatus($orderId) Вывод статуса заказа
 * @method ReturnHTMLOrderStatus($orderId) Возвращение html шаблона статуса заказа
 * @method JsonOrderStatus($orderId) Статус заказа в json формате
 * @method HtmlInJsonOrderStatus($orderId) Статус заказа в json вместе с шаблоном
 *
 *
 * Магические обертки вокруг GetOrderInfo()
 * @method ShowOrderInfo($orderId) Вывод информации о заказе
 * @method ReturnHTMLOrderInfo($orderId) Возвращение html шаблона информации о заказе
 * @method JsonOrderInfo($orderId) Информация о заказе в json формате
 * @method HtmlInJsonOrderInfo($orderId) Информация о заказе в json вместе с шаблоном
 *
 * @method ShowReconcile
 * @method ShowJournal
 *
 * Магические обертки вокруг GetLog()
 * @method ShowLog()
 * @method ReturnHTMLLog()
 * @JsonLog()
 * @HtmlInJsonLog()
 *
 */
class GatewayForAdmin
{
    protected $FLAG_HTML;
    protected $FLAG_JSON_RESULT;
    protected $FLAG_HTML_PRINT;
    protected $FLAG_JSON_WITH_HTML;
    protected $FLAG_HTML_RETURN;

    const DEFAULT_PREFIX = 'Get';
    const JSON_RESULT_PREFIX = 'Json';
    const HTML_RESULT_PREFIX = 'Show';
    const HTML_RETURN_PREFIX = 'ReturnHTML';
    const JSON_WITH_HTML_RESULT_PREFIX = 'HtmlInJson';

    static $__callCache = null;
    private $_functionBody = '';
    private $flagConfig;

    function __construct(InterfaceConnectionConfig $connector = null)
    {
        $this->FLAG_HTML = 1;
        $this->FLAG_JSON_RESULT = 2;
        $this->FLAG_HTML_PRINT = 4 | $this->FLAG_HTML;
        $this->FLAG_JSON_WITH_HTML = $this->FLAG_HTML | $this->FLAG_JSON_RESULT;
        $this->FLAG_HTML_RETURN = 8 | $this->FLAG_HTML;

        $this->flagConfig = new BitwiseFlag();

        if ($connector !== null) {
            Core::GetCore()->PlugInCMSConnector($connector);
        }
    }

    public function ConnectToCMS(InterfaceConnectionConfig $connector)
    {
        Core::GetCore()->PlugInCMSConnector($connector);
        return $this;
    }

    /**
     * Создание виртуальных оберток вокруг методов Get* которые в зависимости от приставки Show*\GetJSON_* возвращают результат в определенном формате
     *
     * @param $functionName
     * @param $arguments
     * @return mixed|null
     */
    public function __call($functionName, $arguments)
    {
        $var_0 = null;
        $var_1 = null;
        $var_2 = null;

        if (count($arguments) == 1 && is_array($arguments[0])) {
            //Разворачивание аргументов если мы передавали их по цепочке
            $arguments = $arguments[0];

            //PHP >5.6 можно разворачивать аргументы через ...$arguments при передаче дальше по цепочке
            extract($arguments, EXTR_PREFIX_ALL, 'var');
        } else {
            extract($arguments, EXTR_PREFIX_ALL, 'var');
        }

        if (static::$__callCache == null) {
            $this->_fillCallCache();
        }

        if (isset(static::$__callCache[$functionName])) {
            $getFunctionName = static::$__callCache[$functionName];

            if (strpos($functionName, self::JSON_RESULT_PREFIX) === 0) {
                $this->flagConfig->setFlag($this->FLAG_JSON_RESULT);
            } elseif (strpos($functionName, self::JSON_WITH_HTML_RESULT_PREFIX) === 0) {
                $this->flagConfig->setFlag($this->FLAG_JSON_WITH_HTML);
            } elseif (strpos($functionName, self::HTML_RESULT_PREFIX) === 0) {
                $this->flagConfig->setFlag($this->FLAG_HTML_PRINT);
            } elseif (strpos($functionName, self::HTML_RETURN_PREFIX) === 0) {
                $this->flagConfig->setFlag($this->FLAG_HTML_RETURN);
            } else {
                return null;
            }

            //Получаем имя реализованной функции для вызова
            $this->_functionBody = substr($getFunctionName, strlen(self::DEFAULT_PREFIX));

            $resultDataContainer = $this->$getFunctionName(${'var_0'}, ${'var_1'}, ${'var_2'});

            return $this->PrepareCallResult($resultDataContainer);
        }

        return NULL;
    }

    private function PrepareCallResult(DataContainer $resultDataContainer)
    {
        $viewDir = Core::GetCore()->GetViewsDir();

        //Обработка HTML шаблона
        if ($this->flagConfig->isFlagSet($this->FLAG_HTML)) {
            global $view_data;
            $view_data = $resultDataContainer->GetResultArray();

            $templateFile = $viewDir . 'ubrr.view.' . strtolower($this->_functionBody) . '.php';

            if ($resultDataContainer->isSuccess() && file_exists($templateFile)) {

                //Буферизация вывода шаблона
                ob_start();
                include_once($templateFile);
                $html_output = ob_get_clean(); //ob_get_contents + ob_end_clean

                $resultDataContainer->SetResultRecord('html', $html_output);
            } else {

                //TODO: обрабатывать ошибки и если они есть выводить строго определенный шаблон вывода ошибок
                print_r($resultDataContainer->GetErrors());

                return false;
            }
        }

        //Вывод шаблона в консоль
        if ($this->flagConfig->isFlagSet($this->FLAG_HTML_PRINT)) {
            $arResult = $resultDataContainer->GetResultArray();
            echo $arResult['html'];
        }
        if ($this->flagConfig->isFlagSet($this->FLAG_HTML_RETURN)) {
            $arResult = $resultDataContainer->GetResultArray();
            return $arResult['html'];
        }

        if ($this->flagConfig->isFlagSet($this->FLAG_JSON_RESULT)) {
            header('Content-Type: application/json');
            echo json_encode($resultDataContainer->GetResultArray());
        }

        return $resultDataContainer;
    }

    protected function _fillCallCache()
    {
        static::$__callCache = Array();

        //$methodsList = get_class_methods(__CLASS__);
        $class = new \ReflectionClass(__CLASS__);
        $publicMethodList = $class->getMethods(\ReflectionMethod::IS_PUBLIC);

        foreach ($publicMethodList as $reflectionMethod) {
            $methodName = $reflectionMethod->name;
            if (substr($methodName, 0, 3) == 'Get') {
                $methodNameBody = substr($methodName, 3);

                //static::$__callCache[self::DATA_CONTAINER_RESULT_PREFIX . $methodNameBody] = $methodName;
                static::$__callCache[self::HTML_RESULT_PREFIX . $methodNameBody] = $methodName;
                static::$__callCache[self::JSON_WITH_HTML_RESULT_PREFIX . $methodNameBody] = $methodName;
                static::$__callCache[self::JSON_RESULT_PREFIX . $methodNameBody] = $methodName;
                static::$__callCache[self::HTML_RETURN_PREFIX . $methodNameBody] = $methodName;
            }
        }
    }

    public function SetupAndGetOrderInformation($TWPG_OrderId, $SessionId)
    {
        $serverResponse = ServerConnector::GetOrderInformation($TWPG_OrderId, $SessionId);

        if ($serverResponse->isSuccess()) {
            $orderInfo = $serverResponse->XMLNestedFields->Response->Order->row;

            $parsedResponse['ID'] = intval($orderInfo->id[0]);
            $parsedResponse['SessionID'] = strval($orderInfo->SessionID[0]);
            $parsedResponse['CreateDate'] = strval($orderInfo->createDate[0]);
            $parsedResponse['LastUpdateDate'] = strval($orderInfo->lastUpdateDate[0]);
            $parsedResponse['PayDate'] = strval($orderInfo->payDate[0]);
            $parsedResponse['MerchantID'] = strval($orderInfo->MerchantID[0]);
            $parsedResponse['Amount'] = floatval($orderInfo->Amount[0]) / 100;
            $parsedResponse['Currency'] = strval($orderInfo->Currency[0]);
            $parsedResponse['OrderLanguage'] = strval($orderInfo->OrderLanguage[0]);
            $parsedResponse['Description'] = strval($orderInfo->Description[0]);
            $parsedResponse['ApproveURL'] = strval($orderInfo->ApproveURL[0]);
            $parsedResponse['CancelURL'] = strval($orderInfo->CancelURL[0]);
            $parsedResponse['DeclineURL'] = strval($orderInfo->DeclineURL[0]);
            $parsedResponse['OrderStatus'] = TypeEnums::GetLabel_OrderStatus($orderInfo->Orderstatus[0]);
            $parsedResponse['RefundAmount'] = strval($orderInfo->RefundAmount[0]);
            $parsedResponse['RefundCurrency'] = strval($orderInfo->RefundCurrency[0]);
            $parsedResponse['ExtSystemProcess'] = strval($orderInfo->ExtSystemProcess[0]);
            $parsedResponse['OrderType'] = TypeEnums::GetLabel_OrderType($orderInfo->OrderType[0]);
            $parsedResponse['Fee'] = strval($orderInfo->Fee[0]);
            $parsedResponse['RefundDate'] = strval($orderInfo->RefundDate[0]);
            $parsedResponse['TWODate'] = strval($orderInfo->TWODate[0]);
            $parsedResponse['TWOTime'] = strval($orderInfo->TWOTime[0]);

            $parsedResponse['Operations'] = Array();
            $arOperations = $orderInfo->OrderOperations->row;
            foreach ($arOperations as $operation) {
                $formattedOperation = Array(
                    'PackageId' => strval($operation->PackageId[0]),
                    'CreateDate' => strval($operation->createDate[0]),
                    'MerchantID' => strval($operation->MerchantID[0]),
                    'OperType' => strval($operation->OperType[0]),
                    'OperName' => TypeEnums::GetLabel_OperName($operation->OperName[0]),
                    'OrderId' => strval($operation->OrderId[0]),
                    'Amount' => floatval($operation->Amount[0]) / 100,
                    'Currency' => strval($operation->Currency[0]),
                    'Approval' => strval($operation->Approval[0]),
                    'TwoId' => strval($operation->twoId[0]),
                );

                $parsedResponse['Operations'][] = $formattedOperation;
            }

            $serverResponse->SetParsedResponse($parsedResponse, 'orderinfo');
        }

        return $serverResponse;
    }

    private function GetOrdersList($pageNum = 1, $perPageCount = 100)
    {
        $connector = Core::GetCore()->GetCMSConnector();

        return $connector->GetOrdersList($pageNum, $perPageCount);
    }

    //region Методы с ВОЛШЕБНЫМИ собратьями :)
    public function GetOrders($pageNum = 1, $perPageCount = 0)
    {
        $arResult = $this->GetOrdersList($pageNum, $perPageCount);
        $resultDataContainer = new DataContainer();
        $resultDataContainer->SetResultRecord('order_list', $arResult);
        $resultDataContainer->setSuccess(); //Пустой результирующий массив это не ошибка!

        return $resultDataContainer;
    }

    /**
     * @param int $TWPG_OrderId
     * @return null|DataContainer
     */
    public function GetOrderStatus($TWPG_OrderId = -1)
    {
        if ($TWPG_OrderId > 0) {
            $orderRow = Core::GetCore()->GetCMSConnector()->GetOrderDataByUBRIROrderID($TWPG_OrderId);
            $TWPG_SessionId = $orderRow['session_id'];

            $paymentPanel = new GatewayPayment();
            $resultDataContainer = $paymentPanel->SetupAndCheckOrderStatus($TWPG_OrderId, $TWPG_SessionId);
            $orderStatusCode = $resultDataContainer->getFieldValue('OrderStatus');
            $orderStatus = TypeEnums::GetLabel_OrderStatus($orderStatusCode);

            $resultDataContainer->SetResultRecord('order_status', Array('code' => $orderStatusCode))
                ->SetResultRecord('label', $orderStatus);

            if (!$resultDataContainer->isSuccess()) {
                $shopId = $this->GetShopId();
                $operation = $resultDataContainer->getFieldValue('Operation');
                if (strpos($operation, '!=') !== false && strpos($operation, $shopId) !== false) {
                    $resultDataContainer->AddError('Заказ был оформлен с другим номером ИМ');
                    $resultDataContainer->SetResultRecord('order_status', Array('code' => $orderStatusCode))
                        ->SetResultRecord('label', 'Заказ был оформлен с другим номером ИМ');

                }
            }
        } else {
            $resultDataContainer = new DataContainer();
            $resultDataContainer->AddError('Недостаточно параметров для выполнения запроса');
        }

        return $resultDataContainer;
    }

    /**
     * @param int $TWPG_OrderId
     * @return null|DataContainer
     */
    public function GetOrderInfo($TWPG_OrderId = -1)
    {
        if ($TWPG_OrderId > 0) {
            $connector = Core::GetCore()->GetCMSConnector();

            $orderRow = $connector->GetOrderDataByUBRIROrderID($TWPG_OrderId);
            $TWPG_SessionId = $orderRow['session_id'];

            $responseData = $this->SetupAndGetOrderInformation($TWPG_OrderId, $TWPG_SessionId);
            if (!$responseData->isSuccess()) {
                $shopId = $connector->GetShopId();
                $operation = $responseData->getFieldValue('Operation');
                if (strpos($operation, '!=') !== false && strpos($operation, $shopId) !== false) {
                    $responseData->AddError('Заказ был оформлен с другим номером ИМ');
                }
            } else {
                $parsedResponse = $responseData->GetParsedResponse();

                $responseData->SetResultRecord('order_info', $parsedResponse['orderinfo']);
            }
        } else {
            $responseData = new DataContainer();
            $responseData->AddError('Недостаточно параметров для выполнения запроса');
        }

        return $responseData;
    }

    private function RetrieveJournal()
    {
        $responseData = ServerConnector::GetJournal();
        if ($responseData->isSuccess()) {
            $journalRecordCount = intval($responseData->XMLNestedFields->Response->Operations->Count[0]);

            $parsedResponse = Array('journal_record_count' => $journalRecordCount);
            $parsedResponse['operations'] = Array();

            $arOrders = $responseData->XMLNestedFields->Response->Operations->Order;
            foreach ($arOrders as $order) {
                $parsedOrder = Array(
                    'id' => intval($order['ID']),
                    'time' => $this->ConvertTime($order->Time[0]),
                    'amount' => (($order->Amount[0]) / 100),
                    'currency' => Helpers::DecodeCurrency($order->Currency[0]),
                    'type' => TypeEnums::GetLabel_OrderType($order->Type[0]),
                    'status' => TypeEnums::GetLabel_OrderStatus($order->Status[0]),
                    'two_id' => intval($order->twoId[0]),
                    'oper_type' => intval($order->OperType[0]),
                    'open_name' => TypeEnums::GetLabel_OperName($order->OperName[0])
                );
                $parsedResponse['operations'][] = $parsedOrder;
            }

            $responseData->SetParsedResponse($parsedResponse, 'journal');
        }

        return $responseData;
    }


    public function GetJournal()
    {
        $responseData = $this->RetrieveJournal();
        if (!$responseData->isSuccess()) {
            $shopId = $this->GetShopId();
            $operation = $responseData->getFieldValue('Operation');
            if (strpos($operation, '!=') !== false && strpos($operation, $shopId) !== false) {
                $responseData->AddError('Заказ был оформлен с другим номером ИМ');
            }
        } else {
            $responseData->CopyFieldsToResultArray();
        }

        return $responseData;
    }

    //endregion

    public function GetLog()
    {
        $core = Core::GetCore();
        return $core->GetLog();
    }

    /**
     * @return DataContainer
     */
    public static function ReturnEmptyResult()
    {
        $emptyDataContainer = new DataContainer();

        return $emptyDataContainer;
    }
}