<?php

use ISYS\UBRIR\OpencartAdmin;
use ISYS\UBRIR\OpencartPayment;
use ISYS\UBRIR\OpencartUBRiRConnector;
use UBRIR\Core;
use UBRIR\NewCertificates;

$currentDir = dirname(__FILE__) . '/';
include_once($currentDir . 'classes/_autoload.php');
include_once($currentDir . 'ubrr.core/_autoload.php');

class ControllerExtensionPaymentUbrr extends Controller
{
    private $connector;
    private $error = array();
    private $version = '2.0';

    public function index()
    {
        $this->connector = new OpencartUBRiRConnector($this);

        $this->CheckDBStructure();
        $data = $this->LoadLocalizationData();
        $this->document->setTitle($this->language->get('heading_title'));

        $data['action'] = $this->makeUrl('extension/payment/ubrr');
        $data['cancel'] = $this->GetPaymentListLink();
        $data['terms'] = '';
        $data['diagnosticReport'] = '';
        $data['css_files'] = Array();
        $data['js_files'] = Array();

        $data = array_merge($data, $this->ParseFormData());
        $data['ajax_response'] = $this->CheckAjaxRequests();

        if (empty($data['ajax_response'])) { //Небольшая оптимизация, если возрвращаем ajax - то нам нет нужды перерисовывать остальные части админки
            if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
                //Не используем штатный способ сохранения параметрво потому у нас несколько форм на разных закладках + есть свои скрытые поля
                //$this->load->model('setting/setting');
                //$this->model_setting_setting->editSetting('ubrr', $this->request->post);
                $this->connector->SaveConfigFormParams($this->request->post);
                $this->session->data['success'] = sprintf($this->language->get('text_success'), $this->language->get('heading_title'));
                $this->response->redirect($this->GetPaymentListLink());
            }

            $data['header'] = $this->load->controller('common/header');
            $data['column_left'] = $this->load->controller('common/column_left');
            $data['footer'] = $this->load->controller('common/footer');

            $cmsModule = new OpencartAdmin($this);
            $data['cms_module'] = $cmsModule;

            //region UBRIR Check

            $core = Core::GetCore();
            $cmsNavigator = new \ISYS\UBRIR\OpencartNavigator();
            $ubrirDiagnostic = new \UBRIR\Diagnostics($this->connector);
            $ubrirDiagnostic->UseNavigator($cmsNavigator);

            $data['diagnosticReport'] = $ubrirDiagnostic->GetDiagnosticPage();

            $data['css_files'] = Array();
            $data['js_files'] = Array();

            $ubrrCoreViewDir = '/' . trim(substr($core->GetViewsDir(), strlen($_SERVER['DOCUMENT_ROOT'])), "/ \t\n\r\0\x0B");
            $moduleViewDir = '/' . trim(substr(dirname(__FILE__) . '/views/', strlen($_SERVER['DOCUMENT_ROOT'])), "/ \t\n\r\0\x0B");
            $data['css_files'][] = $ubrrCoreViewDir . '/css/ubrir.css';
            $data['css_files'][] = $moduleViewDir . '/css/style.css';

            $data['js_files'][] = $ubrrCoreViewDir . '/js/ubrir.admin.js';
            $data['js_files'][] = $ubrrCoreViewDir . '/js/ubrir.admin.orders.js';
            $data['js_files'][] = $moduleViewDir . '/js/opencart_ubrir_module.js?v=3';

            //$stylesFile = dirname(__FILE__) . '/views/css/ubrir_opencart.css';
            //$data['module_css_url'] = substr($stylesFile, strlen($_SERVER['DOCUMENT_ROOT']));

            $data['action_clean_debug_log'] = $this->makeUrl('extension/payment/ubrr/clean_debug_log');
            //      $data['cms_module'] = $cmsModule;
        } else {
            $data['header'] = '';
            $data['column_left'] = '';
            $data['footer'] = '';
        }

        $viewResult = \ISYS\UBRIR\OpencartCompatible::GetView('extension/payment/ubrr', $data, $this);

        $this->response->setOutput($viewResult);
    }

    public function GetPaymentListLink()
    {
        if (version_compare(VERSION, '3.0.0', '<')) {
            return $this->makeUrl('extension/extension', 'type=payment');
        } else {
            return $this->makeUrl('marketplace/extension', 'type=payment');
        }
    }

    protected function CheckDBStructure()
    {
        //TODO: можно хранить в конфиге что структура была проверена - он все равно каждый раз собирается из БД, чтобы не делать дополнительные запросы
        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "twpg_orders` (`shoporderid` VARCHAR(11), `OrderID`
                INT(11) NOT NULL,`SessionID` VARCHAR(40) NOT NULL) ENGINE=InnoDB DEFAULT CHARSET=utf8");
    }

    protected function CheckAjaxRequests()
    {
        $ajax_response = '';

        $ajaxType = '';
        if (isset($_REQUEST['ajax_type']))
            $ajaxType = $_REQUEST['ajax_type'];

        switch ($ajaxType) {
            case 'ajax_action':
                {
                    $actionName = $_REQUEST['action_name'];

                    switch ($actionName) {
                        case 'get_file_request_csr':
                            {
                                $connector = new \ISYS\UBRIR\OpencartUBRiRConnector($this);
                                $cmsNavigator = new \ISYS\UBRIR\OpencartNavigator();
                                $ubrirCertificates = new \UBRIR\NewCertificates($connector);
                                $ubrirCertificates->UseNavigator($cmsNavigator);

                                $ubrirCertificates->ReturnRequestFile();
                                die();
                            }
                            break;
                    }
                }
                break;
            case 'tab_refresh':
                {
                    global $result, $openCartController;

                    $openCartController = $this;

                    $result = 'ooups..';

                    $ajaxDir = dirname(__FILE__) . '/ajax/';
                    $files = scandir($ajaxDir);
                    $tabId = $_REQUEST['tab_id'];

                    $ajax_page = substr($tabId, strlen('#tab_')) . '.php';

                    if (in_array($ajax_page, $files)) {
                        include_once($ajaxDir . $ajax_page);
                    }

                    if ($result === false) {
                        $result = '<br/>Ошибки при выполнении запроса!';
                    }

                    return $result;
                }
                break;

            case 'clear_log':
                {
                    $panelAdmin = new OpencartAdmin($this);
                    $data['debug_log'] = $panelAdmin->CleanLog();
                    return $panelAdmin->ReturnHTMLLog();
                }
                break;
        }

        return $ajax_response;
    }

    protected
    function LoadLocalizationData()
    {
        $data = Array();

        $this->load->language('extension/payment/ubrr');
        $this->load->model('localisation/language');
        $this->load->model('localisation/order_status');

        $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

        $data['lang'] = $this->language;

        //Загрузка названия для цепочки навигации
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->makeUrl('common/dashboard')
        );

        $paymentListHref = $this->GetPaymentListLink();
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_payment'),
            'href' => $paymentListHref
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->makeUrl('extension/payment/ubrr')
        );

        return $data;
    }

    protected function ParseFormData()
    {
        $arDefaultConfig = OpencartUBRiRConnector::GetDefaultConfig();

        $prefix = '';
        if (version_compare(VERSION, '3.0.0', '>=')) {
            $prefix = 'payment_';
        }

        $data = Array();
        foreach ($arDefaultConfig as $code => $defaultValue) {
            if (isset($this->request->post[$code])) {
                $data[$code] = $this->request->post[$code];
            } else {
                $data[$code] = $this->config->get($prefix . $code);
            }
        }

        $data['error_ubrr_merchant_id'] = '';
        if (isset($this->request->post['ubrr_merchant_id'])) {
            $data['ubrr_merchant_id'] = $this->request->post['ubrr_merchant_id'];
        } else {
            if (empty($this->config->get($prefix . 'ubrr_merchant_id'))) {
                $data['error_ubrr_merchant_id'] = $this->language->get('error_ubrr_merchant_id');
            }
            $data['ubrr_certificate_email_receivers'] = $this->config->get($prefix . 'ubrr_certificate_email_receivers');
        }

        $data['error_ubrr_certificate_email_receivers'] = '';
        if (isset($this->request->post['ubrr_certificate_email_receivers'])) {
            $data['ubrr_certificate_email_receivers'] = $this->request->post['ubrr_certificate_email_receivers'];
        } else {
            if (empty($this->config->get($prefix . 'ubrr_certificate_email_receivers'))) {
                $data['error_ubrr_certificate_email_receivers'] = $this->language->get('error_ubrr_certificate_email_receivers');
            }
            $data['ubrr_certificate_email_receivers'] = $this->config->get($prefix . 'ubrr_certificate_email_receivers');
        }

        return $data;
    }

    protected
    function validate()
    {
        if (!$this->validatePermission()) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        return !$this->error;
    }

    protected
    function validatePermission()
    {
        return $this->user->hasPermission('modify', 'extension/payment/ubrr');
    }

    protected
    function makeUrl($route, $url = '')
    {
        if (isset($this->session->data['token'])) {
            return str_replace('&amp;', '&', $this->url->link($route, $url . '&token=' . $this->session->data['token'], 'SSL'));
        } else {
            return str_replace('&amp;', '&', $this->url->link($route, $url . '&user_token=' . $this->session->data['user_token'], 'SSL'));
        }
    }
}

?>